﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Diagnostics;
using Windows.UI.Core;
using Windows.System;
using System.Globalization;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace InsuranceApp
{
    /// <summary>
    /// Insurance Calculator Page
    /// </summary>
    public sealed partial class MainPage : Page
    {
        ServerCommunication com;
        EventListener listener;
        public MainPage()
        {
            this.InitializeComponent();
            com = new ServerCommunication();
            // Event listener for recieving info from async functions
            listener = new EventListener(com, this);

            this.SizeChanged += MainPage_SizeChanged;
            this.calc_grid_1.Width = Window.Current.Bounds.Width;
            this.calc_grid_2.Width = Window.Current.Bounds.Width;
            this.calc_grid_3.Width = Window.Current.Bounds.Width;
            this.calc_grid_1.Height = Window.Current.Bounds.Height;
            this.calc_grid_2.Height = Window.Current.Bounds.Height;
            this.calc_grid_3.Height = Window.Current.Bounds.Height;

            // Set initial values
            // buildYear
            this.calc_buildYear_dp.MaxYear = DateTimeOffset.Now;
            this.calc_buildYear_dp.MinYear = DateTimeOffset.Parse("1000.01.01");
            // StartDate
            this.calc_insuranceStartDate_dp.MinYear = DateTimeOffset.Now;
            // validTo
            this.calc_validTo_dp.MinYear = DateTimeOffset.Now;

            // Hide Ordering related things
            /**this.calc_name_tb.Visibility = Visibility.Collapsed;
            this.calc_surName_tb.Visibility = Visibility.Collapsed;
            this.calc_order_btn.Visibility = Visibility.Collapsed;
            this.calc_validTo_dp.Visibility = Visibility.Collapsed;
            this.calc_order_btn.Visibility = Visibility.Collapsed;
            this.calc_orderConfirm_btn.Visibility = Visibility.Collapsed;
            this.calc_delete_btn.Visibility = Visibility.Collapsed;**/
        }

        // Show price on page
        public void setPrice(string price, string currency)
        {
            switch (currency)
            {
                case "EUR":
                    currency = "€";
                    break;
                case "USD":
                    currency = "$";
                    break;
                default:
                    currency = String.Empty;
                    break;
            }
            this.calc_price_tb.Text = price + currency;
        }

        // On confirmation
        public void orderConfirmation(string id)
        {
            this.calc_notificationBox_tb.Text = "Confirmed";
        }
        // On delete
        public void orderDeleted()
        {
            this.calc_notificationBox_tb.Text = "Application deleted";
        }

        // reset button
        private void calc_back_btn_Click(object sender, RoutedEventArgs e)
        {
            com.resetData();

            // Reset values
            // buildYear
            this.calc_buildYear_dp.Date = DateTimeOffset.Now;
            // StartDate
            this.calc_insuranceStartDate_dp.Date = DateTimeOffset.Now;
            // validTo
            this.calc_validTo_dp.Date = DateTimeOffset.Now;
            // postalCode
            this.calc_postalCode_tb.Text = String.Empty;
            // address
            this.calc_address_tb.Text = String.Empty;
            // area
            this.calc_area_tb.Text = String.Empty;
            // houseType
            this.calc_houseType_cb.SelectedIndex = 0;
            // billingPeriod
            this.calc_billingPeriod_cb.SelectedIndex = 2;
            // currency
            this.calc_currency_cb.SelectedIndex = 0;
            // price
            this.calc_price_tb.Text = String.Empty;
            // name
            this.calc_name_tb.Text = String.Empty;
            // surName
            this.calc_surName_tb.Text = String.Empty;

            // Empty notificationbox
            this.calc_notificationBox_tb.Text = String.Empty;

            // Hide Ordering related things
            /**
            this.calc_name_tb.Visibility = Visibility.Collapsed;
            this.calc_surName_tb.Visibility = Visibility.Collapsed;
            this.calc_order_btn.Visibility = Visibility.Collapsed;
            this.calc_validTo_dp.Visibility = Visibility.Collapsed;
            this.calc_order_btn.Visibility = Visibility.Collapsed;
            this.calc_orderConfirm_btn.Visibility = Visibility.Collapsed;
            this.calc_delete_btn.Visibility = Visibility.Collapsed;**/
        }

        // Calculate button
        private void calc_calc_btn_Click(object sender, RoutedEventArgs e)
        {
            string area = String.Empty;
            string houseType = String.Empty;
            string billingPeriod = String.Empty;
            string currency = String.Empty;
            int postalCode_i = 0;
            double area_d = 0.0;
            bool input_okay = true;
            string bad_input = "Bad input";

            // Read Postal Code
            try
            {
                postalCode_i = Convert.ToInt32(calc_postalCode_tb.Text);
            }
            catch (FormatException)
            {
                calc_postalCode_tb.Text = bad_input;
            }
            catch (OverflowException)
            {
                calc_postalCode_tb.Text = bad_input;
            }
            string postalCode = postalCode_i.ToString();

            // Read Address
            string address = this.calc_address_tb.Text;

            // Read Area
            try
            {
                calc_area_tb.Text = calc_area_tb.Text.Replace(',', '.');
                area_d = System.Double.Parse(calc_area_tb.Text, CultureInfo.InvariantCulture);
            }
            catch (FormatException)
            {
                calc_area_tb.Text = bad_input;
            }
            catch (OverflowException)
            {
                calc_area_tb.Text = bad_input;
            }
            area = String.Format("{0:0.0}", area_d);

            // Read Build Year
            string buildYear = this.calc_buildYear_dp.Date.ToString("yyyy"); ;

            // Read Start Date
            string insuranceStartDate = this.calc_insuranceStartDate_dp.Date.ToString("dd.MM.yyyy");

            // Read House Type
            switch (calc_houseType_cb.SelectedIndex)
            {
                case 0:
                    houseType = "HOUSE";
                    break;
                case 1:
                    houseType = "APARTMENT";
                    break;
                case 2:
                    houseType = "ROWHOUSE";
                    break;
                case 3:
                    houseType = "PAIRHOUSE";
                    break;
                case 4:
                    houseType = "SUMMERHOUSE";
                    break;
                case 5:
                    houseType = "SAUNA";
                    break;
                default:
                    houseType = this.calc_houseType_cb.SelectedValue.ToString();
                    break;
            }

            // Read Billing Period
            switch (calc_billingPeriod_cb.SelectedIndex)
            {
                case 0:
                    billingPeriod = "MONTH";
                    break;
                case 1:
                    billingPeriod = "QUARTER";
                    break;
                case 2:
                    billingPeriod = "YEAR";
                    break;
                default:
                    houseType = this.calc_billingPeriod_cb.SelectedValue.ToString();
                    break;
            }

            // Read Currency
            switch (calc_currency_cb.SelectedIndex)
            {
                case 0:
                    currency = "EUR";
                    break;
                case 1:
                    currency = "USD";
                    break;
                default:
                    currency = this.calc_currency_cb.SelectedValue.ToString();
                    break;
            }

            // Check for empty input
            if (String.IsNullOrEmpty(postalCode) || postalCode_i == 0)
            {
                input_okay = false;
                this.calc_postalCode_tb.Text = bad_input;
            }
            if (String.IsNullOrEmpty(address))
            {
                input_okay = false;
                this.calc_address_tb.Text = bad_input;
            }
            if (String.IsNullOrEmpty(area))
                input_okay = false;

            if (input_okay)
            {
                com.saveData(postalCode, address, area, buildYear, insuranceStartDate, houseType, billingPeriod, currency);

                com.calculateInsurance();

                this.calc_scrollviewer.ChangeView(Window.Current.Bounds.Width, 0, 1);

                // Show Order button
                //this.calc_order_btn.Visibility = Visibility.Visible;
            }
        }

        // Order confirm button
        private void calc_orderConfirm_btn_Click(object sender, RoutedEventArgs e)
        {
            // Read order info
            string name = this.calc_name_tb.Text;
            string surName = this.calc_surName_tb.Text;
            string validTo = this.calc_validTo_dp.Date.ToString("dd.MM.yyyy");
            // Hopefully place order
            com.orderInsurance(name, surName, validTo);
            // Show delete button
            this.calc_delete_btn.Visibility = Visibility.Visible;
        }

        // Order button
        private void calc_order_btn_Click(object sender, RoutedEventArgs e)
        {
            // Make order info boxes and confirm button visible
            /**this.calc_name_tb.Visibility = Visibility.Visible;
            this.calc_surName_tb.Visibility = Visibility.Visible;
            this.calc_orderConfirm_btn.Visibility = Visibility.Visible;
            this.calc_validTo_dp.Visibility = Visibility.Visible;
            this.calc_order_btn.Visibility = Visibility.Collapsed;**/
            this.calc_scrollviewer.ChangeView(Window.Current.Bounds.Width * 2, 0, 1);
        }

        // Clear textbox on select
        private void calc_textBox_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox tb = (TextBox)sender;
            tb.Text = string.Empty;
            tb.GotFocus -= calc_textBox_GotFocus;
        }

        // Delete button
        private void calc_delete_btn_Click(object sender, RoutedEventArgs e)
        {
            // Delete application from remote server
            com.deleteInsurance();
        }

        private void calc_postalCode_tb_KeyDown(object sender, KeyRoutedEventArgs e)
        {
            char key = Convert.ToChar(e.Key);

            //if the key is not 0-9 prevent the event from being handled further
            if (!(key >= '0' && key <= '9'))
                e.Handled = true; 
        }

        void MainPage_SizeChanged(object sender, SizeChangedEventArgs e) 
        {
            this.calc_grid_1.Width = Window.Current.Bounds.Width;
            this.calc_grid_2.Width = Window.Current.Bounds.Width;
            this.calc_grid_3.Width = Window.Current.Bounds.Width;
            this.calc_grid_1.Height = Window.Current.Bounds.Height;
            this.calc_grid_2.Height = Window.Current.Bounds.Height;
            this.calc_grid_3.Height = Window.Current.Bounds.Height;
       }
    }
}
