﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization.Json;
using System.Net.Http;
using System.Diagnostics;
using System.Runtime.Serialization;

namespace InsuranceApp
{
    public delegate void NotificationEventHandler(object sender, EventArgs e);

    // Class for basic data
    [DataContract]
    class InsuranceData
    {
        [DataMember(Order = 0)]
        string postalCode;
        [DataMember(Order = 1)]
        string address;
        [DataMember(Order = 2)]
        string area;
        [DataMember(Order = 3)]
        string buildYear;
        [DataMember(Order = 4)]
        string insuranceStartDate;
        [DataMember(Order = 5)]
        string houseType;
        [DataMember(Order = 6)]
        string billingPeriod;
        [DataMember(Order = 7)]
        string currency;

        public InsuranceData()
        {
        }

        public InsuranceData(string setPostalCode, string setAddress, string setArea, string setBuildYear, string setInsuranceStartDate, string setHouseType, string setBillingPeriod, string setCurrency)
        {
            this.postalCode = setPostalCode;
            this.address = setAddress;
            this.area = setArea;
            this.buildYear = setBuildYear;
            this.insuranceStartDate = setInsuranceStartDate;
            this.houseType = setHouseType;
            this.billingPeriod = setBillingPeriod;
            this.currency = setCurrency;
        }
    }

    // Class for price data
    [DataContract]
    class PriceData
    {
        [DataMember(Name = "price")]
        public string price;
        [DataMember(Name = "currency")]
        public string currency;
        [DataMember(Name = "billingPeriod")]
        public string billingPeriod;

        public string getPrice()
        {
            return price;
        }
        public string getCurrency()
        {
            return currency;
        }
        public string getBillingPeriod()
        {
            return billingPeriod;
        }

        public PriceData()
        { }

        public PriceData(string setPrice, string setCurrency, string setBillingperiod)
        {
            price = setPrice;
            currency = setCurrency;
            billingPeriod = setBillingperiod;
        }
    }

    // Class for return data
    [DataContract]
    class ReturnData
    {
        [DataMember(Name = "postalCode")]
        public string postalCode;
        [DataMember(Name = "address")]
        public string address;
        [DataMember(Name = "area")]
        public string area;
        [DataMember(Name = "buildYear")]
        public string buildYear;
        [DataMember(Name = "insuranceStartDate")]
        public string insuranceStartDate;
        [DataMember(Name = "houseType")]
        public string houseType;
        [DataMember(Name = "billingPeriod")]
        public string billingPeriod;
        [DataMember(Name = "currency")]
        public string currency;
        [DataMember(Name = "price")]
        public PriceData price;
    }

    // Class for order data
    [DataContract]
    class OrderData
    {
        [DataMember(Order = 0)]
        string name;
        [DataMember(Order = 1)]
        string surName;
        [DataMember(Order = 2)]
        string validTo;
        [DataMember(Order = 3)]
        ReturnData pricingParameters;

        public OrderData(string setName, string setSurName, string setValidTo, ReturnData returnData)
        {
            name = setName;
            surName = setSurName;
            validTo = setValidTo;
            pricingParameters = returnData;
        }
    }

    [DataContract]
    class IdData
    {
        [DataMember(Name="id")]
        public string id;
    }

    [DataContract]
    class ErrorData
    {
        [DataMember(Name = "timestamp")]
        public string timestamp;
        [DataMember(Name = "status")]
        public string status;
        [DataMember(Name = "error")]
        public string error;
        [DataMember(Name = "exception")]
        public string exception;
        [DataMember(Name = "message")]
        public string message;
        [DataMember(Name = "path")]
        public string path;
    }

    // Communicates with remote server
    class ServerCommunication
    {
        string url_base;
        string userName;
        string password;
        InsuranceData data;
        HttpClient client;
        ReturnData ret_data;
        IdData id_data;

        public ServerCommunication()
        {
            url_base = "http://185.20.136.51/";
            userName = "LUT";
            password = "0gmsl48hgi_jhfiud76";

            client = new HttpClient();
            string plain = userName + ":" + password;
            string authString = System.Convert.ToBase64String(Encoding.UTF8.GetBytes(plain));
            client.DefaultRequestHeaders.Authorization = new System.Net.Http.Headers.AuthenticationHeaderValue("Basic", authString);
        }
        public void resetData()
        {
            data = null;
            ret_data = null;
            id_data = null;
        }

        // Saves basic data to send to server
        public void saveData(string postalCode, string address, string area, string buildYear, string insuranceStartDate, string houseType, string billingPeriod, string currency)
        {
            data = new InsuranceData(postalCode, address, area, buildYear, insuranceStartDate, houseType, billingPeriod, currency);
        }

        // Gets price from data returned from server
        public string getPrice()
        {
            try
            {
                return ret_data.price.getPrice();
            }
            catch
            {
                return "Error";
            }
        }

        public string getId()
        {
            try
            {
                return id_data.id;
            }
            catch
            {
                return "Error";
            }
        }

        public string getCurrency()
        {
            try
            {
                return ret_data.price.getCurrency();
            }
            catch
            {
                return String.Empty;
            }
    }

        public event NotificationEventHandler NotificationEvent;
        public event NotificationEventHandler ConfirmationEvent;
        public event NotificationEventHandler DeletionEvent;

        protected virtual void OnNotification(EventArgs e)
        {
            if (NotificationEvent != null)
                NotificationEvent(this, e);
        }
        protected virtual void OnConfirmation(EventArgs e)
        {
            if (ConfirmationEvent != null)
                ConfirmationEvent(this, e);
        }
        protected virtual void OnDelete(EventArgs e)
        {
            if (DeletionEvent != null)
                DeletionEvent(this, e);
        }

        // Sends POST to sellertool/prices
        async public void calculateInsurance()
        {

            string url = url_base + "sellertool/prices";
            DataContractJsonSerializer ser = new DataContractJsonSerializer(typeof(InsuranceData));

            System.IO.MemoryStream stream = new System.IO.MemoryStream();
            ser.WriteObject(stream, data);
            string json = Encoding.UTF8.GetString(stream.ToArray(), 0, (int)stream.Length);
            Debug.WriteLine(json);
            StringContent messageContent = new StringContent(json, Encoding.UTF8, "application/json");

            HttpResponseMessage response = await client.PostAsync(url, messageContent);

            var str = await response.Content.ReadAsStreamAsync();

            DataContractJsonSerializer deSer = new DataContractJsonSerializer(typeof(ReturnData));

            try
            {
                ret_data = (ReturnData)deSer.ReadObject(str);
            }
            catch
            {
                Debug.WriteLine("Error in calculateInsurance");
            }

            if(ret_data != null)
                OnNotification(EventArgs.Empty);
        }

        //Sends POST to sellertool/applications
        async public void orderInsurance(string name, string surName, string validTo)
        {
            string url = url_base + "sellertool/applications";

            OrderData order_data = new OrderData(name, surName, validTo, ret_data);

            DataContractJsonSerializer ser = new DataContractJsonSerializer(typeof(OrderData));
            System.IO.MemoryStream stream = new System.IO.MemoryStream();

            ser.WriteObject(stream, order_data);

            string json = Encoding.UTF8.GetString(stream.ToArray(), 0, (int)stream.Length);
            Debug.WriteLine(json);
            StringContent messageContent = new StringContent(json, Encoding.UTF8, "application/json");
            HttpResponseMessage response = await client.PostAsync(url, messageContent);

            var str = await response.Content.ReadAsStreamAsync();

            DataContractJsonSerializer deSer = new DataContractJsonSerializer(typeof(IdData));
            try
            {
                id_data = (IdData)deSer.ReadObject(str);
            }
            catch
            {
                Debug.WriteLine("Error in orderInsurance");
            }

            if (id_data != null)
                OnConfirmation(EventArgs.Empty);
        }

        async public void deleteInsurance()
        {
            try
            {
                if (id_data != null)
                {
                    string url = url_base + "sellertool/applications/" + id_data.id;
                    await client.DeleteAsync(url);
                    HttpResponseMessage response = await client.GetAsync(url);
                    var str = await response.Content.ReadAsStringAsync();
                    id_data = null;

                    if (str.Length == 0)
                        OnDelete(EventArgs.Empty);
                }
            }
            catch
            {
                Debug.WriteLine("Error in deleteInsurance");
            }
        }
    }

    //Event Listener for server communication
    class EventListener
    {
        private ServerCommunication serv;
        MainPage senderPage;

        public EventListener(object setServ, object setSender)
        {
            serv = setServ as ServerCommunication;
            senderPage = setSender as MainPage;
            serv.NotificationEvent += new NotificationEventHandler(servChanged);
            serv.ConfirmationEvent += new NotificationEventHandler(servConfirmed);
            serv.DeletionEvent += new NotificationEventHandler(servDeleted);
        }

        private void servChanged(object sender, EventArgs e)
        {
            senderPage.setPrice(serv.getPrice(), serv.getCurrency());
        }

        private void servConfirmed(object sender, EventArgs e)
        {
            senderPage.orderConfirmation(serv.getId());
        }

        private void servDeleted(object sender, EventArgs e)
        {
            senderPage.orderDeleted();
        }
    }
}
